@@ -121,7 +121,7 @@ struct CodeCompletionItemTester {
   }
 
   //Standard constructor
-  CodeCompletionItemTester(DUContext* context, QString text = "; ", QString followingText = "", SimpleCursor position = SimpleCursor::invalid()) {
+  CodeCompletionItemTester(DUContext* context, QString text = "; ", QString followingText = "", CursorInRevision position = CursorInRevision::invalid()) {
     completionContext = new  T(DUContextPointer(context), text, followingText, position.isValid() ? position : context->range().end);
 
     init();
