@@ -39,20 +39,20 @@ using namespace KDevelop;
 class KDevelop::OverridesPagePrivate
 {
 public:
-    OverridesPagePrivate()
-        : overrides(0)
+    OverridesPagePrivate(ClassGenerator* g)
+        : generator(g), overrides(0)
     {
     }
-
+    ClassGenerator* generator;
     Ui::OverridesDialog* overrides;
     QMultiHash<Identifier, DeclarationPointer> overriddenFunctions;
     QMap<QTreeWidgetItem *, DeclarationPointer> declarationMap;
     QList<DeclarationPointer> chosenOverrides;
 };
 
-OverridesPage::OverridesPage(QWizard* parent)
+OverridesPage::OverridesPage(ClassGenerator* generator, QWizard* parent)
     : QWizardPage(parent)
-    , d(new OverridesPagePrivate)
+    , d(new OverridesPagePrivate(generator))
 {
     setTitle(i18n("Override Methods"));
     setSubTitle( i18n("Select any methods you would like to override in the new class.") );
@@ -132,13 +132,17 @@ void OverridesPage::populateOverrideTree(const QList<DeclarationPointer> & baseL
         //For this internal context get all the function declarations inside the class
         foreach (Declaration * childDeclaration, context->localDeclarations())
         {
-            AbstractFunctionDeclaration * func;
-            if (childDeclaration->type<FunctionType>() &&
-                (func = dynamic_cast<AbstractFunctionDeclaration*>(childDeclaration)) &&
-                func->isVirtual())
-            {
-                // Its a virtual function, add it to the list
-                addPotentialOverride(classItem, DeclarationPointer(childDeclaration));
+            if (AbstractFunctionDeclaration * func = dynamic_cast<AbstractFunctionDeclaration*>(childDeclaration)) {
+                if (func->isVirtual()) {
+                    // Its a virtual function, add it to the list
+                    addPotentialOverride(classItem, DeclarationPointer(childDeclaration));
+                } else if (generator()->directInheritanceList().contains(baseClass)) {
+                    // add ctors of direct parents
+                    ClassFunctionDeclaration* cFunc = dynamic_cast<ClassFunctionDeclaration*>(childDeclaration);
+                    if (cFunc && cFunc->isConstructor()) {
+                        addPotentialOverride(classItem, DeclarationPointer(childDeclaration));
+                    }
+                }
             }
         }
     }
@@ -225,4 +229,10 @@ void OverridesPage::deselectAll()
     }
 }
 
+ClassGenerator* OverridesPage::generator() const
+{
+    return d->generator;
+}
+
+
 #include "overridespage.moc"
