@@ -30,7 +30,6 @@
 #include <kpluginloader.h>
 #include <ktexteditor/view.h>
 #include <ktexteditor/document.h>
-#include <ktexteditor/smartinterface.h>
 #include <KTextEditor/TextHintInterface>
 #include <kactioncollection.h>
 #include <kaboutdata.h>
@@ -63,22 +62,33 @@
 #include <language/interfaces/iquickopen.h>
 #include <interfaces/iplugincontroller.h>
 #include <sublime/mainwindow.h>
+#include <memory>
 
-const unsigned int highlightingTimeout = 150;
+static const unsigned int highlightingTimeout = 150;
+static const float highlightingZDepth = -5000;
 
 using KDevelop::ILanguage;
 using KTextEditor::Attribute;
-using KTextEditor::SmartInterface;
 using KTextEditor::View;
 
 bool toolTipEnabled = true;
 
+// Helper that follows the QObject::parent() chain, and returns the highest widget that has no parent.
 QWidget* masterWidget(QWidget* w) {
   while(w && w->parent() && qobject_cast<QWidget*>(w->parent()))
     w = qobject_cast<QWidget*>(w->parent());
   return w;
 }
 
+// Helper that determines the context to use for highlighting at a specific position
+DUContext* contextForHighlightingAt(const SimpleCursor& position, TopDUContext* topContext)
+{
+  DUContext* ctx = topContext->findContextAt(topContext->transformToLocalRevision(position));
+  while(ctx && (ctx->type() == DUContext::Template || ctx->type() == DUContext::Helper || ctx->localScopeIdentifier().isEmpty()) && ctx->parentContext())
+    ctx = ctx->parentContext();
+  return ctx;
+}
+
 class ContextBrowserViewFactory: public KDevelop::IToolViewFactory
 {
 public:
@@ -169,7 +179,6 @@ K_EXPORT_PLUGIN(ContextBrowserFactory(KAboutData("kdevcontextbrowser","kdevconte
 
 ContextBrowserPlugin::ContextBrowserPlugin(QObject *parent, const QVariantList&)
     : KDevelop::IPlugin(ContextBrowserFactory::componentData(), parent)
-    , m_backupsMutex(QMutex::Recursive)
     , m_viewFactory(new ContextBrowserViewFactory(this)), m_lastInsertionDocument(0)
 {
   core()->uiController()->addToolView(i18n("Code Browser"), m_viewFactory);
@@ -191,30 +200,6 @@ ContextBrowserPlugin::ContextBrowserPlugin(QObject *parent, const QVariantList&)
 
 ContextBrowserPlugin::~ContextBrowserPlugin()
 {
-  foreach (KTextEditor::SmartRange* range, m_watchedRanges)
-    range->removeWatcher(this);
-}
-
-void ContextBrowserPlugin::watchRange(KTextEditor::SmartRange* range)
-{
-  KTextEditor::SmartInterface* smart = dynamic_cast<KTextEditor::SmartInterface*>(range->document());
-  Q_ASSERT(smart);
-  QMutexLocker lock(smart->smartMutex());
-
-  if (range->watchers().contains( this ))
-    return;
-
-  range->addWatcher(this);
-  m_watchedRanges.insert(range);
-}
-
-void ContextBrowserPlugin::ignoreRange(KTextEditor::SmartRange* range)
-{
-  if (!range->watchers().contains( this ))
-    return;
-
-  range->removeWatcher(this);
-  m_watchedRanges.remove(range);
 }
 
 void ContextBrowserPlugin::unload()
@@ -222,32 +207,6 @@ void ContextBrowserPlugin::unload()
   core()->uiController()->removeToolView(m_viewFactory);
 }
 
-void ContextBrowserPlugin::rangeDeleted( KTextEditor::SmartRange *range ) {
-  QMutexLocker lock(&m_backupsMutex);
-  m_backups.remove( range );
-  m_watchedRanges.remove(range);
-
-  for(QMap<View*, KTextEditor::SmartRange*>::iterator it = m_highlightedRange.begin(); it != m_highlightedRange.end(); ++it)
-    if(*it == range) {
-      m_highlightedRange.erase(it);
-      return;
-    }
-}
-
-///@todo this doesn't work, we use TextHintInterface instead atm.
-void ContextBrowserPlugin::mouseEnteredRange( KTextEditor::SmartRange* range, View* view ) {
-  m_mouseHoverCursor = SimpleCursor(range->start());
-  m_mouseHoverDocument = view->document()->url();
-  m_updateViews << view;
-  m_updateTimer->start(1);
-}
-
-void ContextBrowserPlugin::mouseExitedRange( KTextEditor::SmartRange* /*range*/, View* view ) {
-  clearMouseHover();
-  m_updateViews << view;
-  m_updateTimer->start(1); // triggers updateViews()
-}
-
 KDevelop::ContextMenuExtension ContextBrowserPlugin::contextMenuExtension(KDevelop::Context* context)
 {
   KDevelop::ContextMenuExtension menuExt = KDevelop::IPlugin::contextMenuExtension( context );
@@ -326,7 +285,6 @@ void ContextBrowserPlugin::hideTooTip() {
   }
 }
 
-
 void ContextBrowserPlugin::showToolTip(KTextEditor::View* view, KTextEditor::Cursor position) {
   KUrl viewUrl(view->document()->url());
   QList<ILanguage*> languages = ICore::self()->languageController()->languagesForUrl(viewUrl);
@@ -389,58 +347,28 @@ void ContextBrowserPlugin::showToolTip(KTextEditor::View* view, KTextEditor::Cur
   }else{
     kDebug() << "not showing tooltip, no navigation-widget";
   }
-  
 }
 
-
 void ContextBrowserPlugin::clearMouseHover() {
   m_mouseHoverCursor = SimpleCursor::invalid();
   m_mouseHoverDocument.clear();
 }
 
-Attribute::Ptr highlightedUseAttribute(bool /*mouseHighlight*/, bool bold) {
-  if(bold) {
-    static Attribute::Ptr standardBoldAttribute = Attribute::Ptr();
-    if( !standardBoldAttribute ) {
-      standardBoldAttribute= Attribute::Ptr( new Attribute() );
-      standardBoldAttribute->setBackgroundFillWhitespace(true);
-
-      // mixing (255, 255, 0, 100) with white yields this:
-      standardBoldAttribute->setBackground(QColor(251, 250, 150));
-      standardBoldAttribute->setFontBold(true);
-
-      // force a foreground color to overwrite default Kate highlighting, i.e. of Q_OBJECT or similar
-      // foreground color could change, hence apply it everytime
-      standardBoldAttribute->setForeground(QColor(0, 0, 0, 255)); //Don't use alpha here, as kate uses the alpha only to blend with the document background color
-    }
-    return standardBoldAttribute;
-  }else{
-    static Attribute::Ptr standardAttribute = Attribute::Ptr();
-    if( !standardAttribute ) {
-      standardAttribute = Attribute::Ptr( new Attribute() );
-      standardAttribute->setBackgroundFillWhitespace(true);
-      // mixing (255, 255, 0, 100) with white yields this:
-      standardAttribute->setBackground(QColor(251, 250, 150));
-      // force a foreground color to overwrite default Kate highlighting, i.e. of Q_OBJECT or similar
-      // foreground color could change, hence apply it everytime
-      standardAttribute->setForeground(QColor(0, 0, 0, 255)); //Don't use alpha here, as kate uses the alpha only to blend with the document background color
-    }
-    return standardAttribute;
-  }
-}
-
-///NOTE: when this gets used again we have to set an alpha value as well
-/*
-Attribute::Ptr highlightedDeclarationAttribute() {
+Attribute::Ptr highlightedUseAttribute() {
   static Attribute::Ptr standardAttribute = Attribute::Ptr();
   if( !standardAttribute ) {
-    standardAttribute = Attribute::Ptr( new Attribute() );
+    standardAttribute= Attribute::Ptr( new Attribute() );
     standardAttribute->setBackgroundFillWhitespace(true);
-    standardAttribute->setBackground(Qt::red);
+
+    // mixing (255, 255, 0, 100) with white yields this:
+    standardAttribute->setBackground(QColor(251, 250, 150));
+
+    // force a foreground color to overwrite default Kate highlighting, i.e. of Q_OBJECT or similar
+    // foreground color could change, hence apply it everytime
+    standardAttribute->setForeground(QColor(0, 0, 0, 255)); //Don't use alpha here, as kate uses the alpha only to blend with the document background color
   }
   return standardAttribute;
 }
-*/
 
 Attribute::Ptr highlightedSpecialObjectAttribute() {
   static Attribute::Ptr standardAttribute = Attribute::Ptr();
@@ -456,98 +384,41 @@ Attribute::Ptr highlightedSpecialObjectAttribute() {
   return standardAttribute;
 }
 
-void ContextBrowserPlugin::changeHighlight( KTextEditor::SmartRange* range, bool highlight, bool /*declaration*/, bool mouseHighlight ) {
-  if( !range )
-    return;
-
-  QMutexLocker lock(&m_backupsMutex);
-  
-  if( highlight ) {
-/*    if( declaration )
-      attrib = highlightedDeclarationAttribute();
-    else*/
-
-    Attribute::Ptr attrib = highlightedUseAttribute(mouseHighlight, range->attribute() ? range->attribute()->fontBold() : false);
-
-    if( !m_backups.contains(range) ) {
-      m_backups[range] = qMakePair(attrib, range->attribute());
-      watchRange(range);
-    }
-    range->setAttribute(attrib);
-  }else{
-    //Reset old highlighting, if our set attribute is still on the range
-    if( m_backups.contains(range) && m_backups[range].first == range->attribute() ) {
-      range->setAttribute(m_backups[range].second);
-      m_backups.remove(range);
-    }
-    ignoreRange(range);
-  }
-
-}
-
-void ContextBrowserPlugin::changeHighlight( View* view, KDevelop::Declaration* decl, bool highlight, bool mouseHighlight ) {
-  if( !view || !decl || !decl->context() ) {
+void ContextBrowserPlugin::addHighlight( View* view, KDevelop::Declaration* decl ) {
+  if( !view || !decl ) {
     kDebug() << "invalid view/declaration";
     return;
   }
+  
+  ViewHighlights& highlights(m_highlightedRanges[view]);
 
-  KTextEditor::SmartRange* range = decl->smartRange();
-  if( range )
-    changeHighlight( range, highlight, true, mouseHighlight );
+  KDevelop::DUChainReadLocker lock;
 
-
-  QList<KTextEditor::SmartRange*> uses;
+  // Highlight the declaration
+  highlights.highlights << decl->createRangeMoving();
+  highlights.highlights.back()->setAttribute(highlightedUseAttribute());
+  highlights.highlights.back()->setZDepth(highlightingZDepth);
+  
+  // Highlight uses
   {
-    KDevelop::DUChainReadLocker lock( DUChain::lock() );
-    uses = decl->smartUses();
+    QMap< IndexedString, QList< SimpleRange > > currentRevisionUses = decl->usesCurrentRevision();
+    for(QMap< IndexedString, QList< SimpleRange > >::iterator fileIt = currentRevisionUses.begin(); fileIt != currentRevisionUses.end(); ++fileIt)
+    {
+      for(QList< SimpleRange >::const_iterator useIt = (*fileIt).constBegin(); useIt != (*fileIt).constEnd(); ++useIt)
+      {
+        highlights.highlights << PersistentMovingRange::Ptr(new PersistentMovingRange(*useIt, fileIt.key()));
+        highlights.highlights.back()->setAttribute(highlightedUseAttribute());
+        highlights.highlights.back()->setZDepth(highlightingZDepth);
+      }
+    }
   }
-
-  foreach(KTextEditor::SmartRange* range, uses)
-    changeHighlight( range, highlight, false, mouseHighlight );
-
+  
   if( FunctionDefinition* def = FunctionDefinition::definition(decl) )
-    if( def->smartRange() )
-      changeHighlight( def->smartRange(), highlight, false, mouseHighlight );
-}
-
-bool ContextBrowserPlugin::findSpecialObject(View* view, const SimpleCursor& position, ILanguage*& pickedLanguage)
-{
-      SmartInterface* iface = dynamic_cast<SmartInterface*>(view->document());
-      if (!iface) return false;
-
-      if(m_highlightedRange.contains(view)) { // remove old highlighting
-        QMutexLocker lock(iface->smartMutex());
-        //Q_ASSERT(m_highlightedRange[view]->document() == view->document());
-        if (m_highlightedRange[view]->document() == view->document()) {
-          delete m_highlightedRange[view];
-          m_highlightedRange.remove(view);
-      } else {
-          kDebug() << "m_highlightedRange[view]->document() != view->document()";	
-        }
-      }
-
-      KUrl viewUrl = view->document()->url();
-      QList<ILanguage*> languages = ICore::self()->languageController()->languagesForUrl(viewUrl);
-
-      SimpleRange r = SimpleRange::invalid();
-      foreach( ILanguage* language, languages) {
-        r = language->languageSupport()->specialLanguageObjectRange(viewUrl, position);
-        if(r.isValid()) {
-          pickedLanguage = language;
-          break;
-        }
-      }
-
-      if (r.isValid()) {
-        m_highlightedRange[view] = iface->newSmartRange( r.textRange(), m_highlightedRange[view] );//iface->newSmartRange( view->document()->documentRange() );
-        iface->addHighlightToDocument(m_highlightedRange[view]);
-        m_highlightedRange[view]->setAttribute( highlightedSpecialObjectAttribute() );
-        watchRange(m_highlightedRange[view]);
-        return true;
-      } else {
-        pickedLanguage = 0;
-        return false;
-      }
+  {
+    highlights.highlights << def->createRangeMoving();
+    highlights.highlights.back()->setAttribute(highlightedUseAttribute());
+    highlights.highlights.back()->setZDepth(highlightingZDepth);
+  }
 }
 
 Declaration* ContextBrowserPlugin::findDeclaration(View* view, const SimpleCursor& position, bool mouseHighlight)
@@ -569,171 +440,120 @@ Declaration* ContextBrowserPlugin::findDeclaration(View* view, const SimpleCurso
       return foundDeclaration;
 }
 
-bool ContextBrowserPlugin::showDeclarationView(View* view, const SimpleCursor& position, Declaration* foundDeclaration, DUContext* ctx)
+ContextBrowserView* ContextBrowserPlugin::browserViewForTextView(View* view)
 {
-        Q_UNUSED(position);
-        Q_ASSERT(foundDeclaration);
-        bool success = false;
-        foreach(ContextBrowserView* contextView, m_views) {
-          if(masterWidget(contextView) == masterWidget(view)) {
-              contextView->setDeclaration(foundDeclaration, ctx->topContext());
-              success = true;
-          }
-        }
-        return success;
-}
-
-bool ContextBrowserPlugin::showSpecialObjectView(View* view, const SimpleCursor& position, ILanguage* pickedLanguage, DUContext* /*ctx*/)
-{
-        //Q_ASSERT(pickedLanguage != 0); // specialObject was found, pickedLanguage must have been set
-        if (!pickedLanguage) {
-          kDebug() << "Special's object language turned null.";
-          return false;
-        }
-        bool success = false;
-        foreach(ContextBrowserView* contextView, m_views) {
-          if(masterWidget(contextView) == masterWidget(view)) {
-              ILanguageSupport* ls = pickedLanguage->languageSupport();
-              QWidget* w = ls->specialLanguageObjectNavigationWidget(view->document()->url(), position);
-              contextView->setSpecialNavigationWidget(w);
-              success = true;
-            }
-        }
-        return success;
-}
-
-void ContextBrowserPlugin::showContextView(View* view, const SimpleCursor& position, DUContext* ctx)
-{
-    Q_UNUSED(position);
-    Q_ASSERT(ctx);
-    foreach(ContextBrowserView* contextView, m_views) {
-      if(masterWidget(contextView) == masterWidget(view)) {
-        contextView->setContext(SpecializationStore::self().applySpecialization(ctx, ctx->topContext()));
-      }
+  foreach(ContextBrowserView* contextView, m_views) {
+    if(masterWidget(contextView) == masterWidget(view)) {
+      return contextView;
     }
-}
-
-DUContext* contextAt(const SimpleCursor& position, TopDUContext* topContext)
-{
-      DUContext* ctx = topContext->findContextAt(position);
-      while(ctx && (ctx->type() == DUContext::Template || ctx->type() == DUContext::Helper || ctx->localScopeIdentifier().isEmpty()) && ctx->parentContext())
-        ctx = ctx->parentContext();
-      return ctx;
-}
-
-
-void ContextBrowserPlugin::unHighlightAll(KTextEditor::View* selectView)
-{
-    foreach(KTextEditor::View* view, m_highlightedDeclarations.keys()) {
-      if(selectView && selectView != view)
-        continue;
+  }
       
-      KTextEditor::SmartInterface* smart = dynamic_cast<KTextEditor::SmartInterface*>(view->document());
-    
-      QMutexLocker smartLock(smart->smartMutex());
-      QMutexLocker lock(&m_backupsMutex);
-
-      QMap< KTextEditor::SmartRange*, QPair< Attribute::Ptr, Attribute::Ptr > >::iterator it = m_backups.begin();
-      while(it != m_backups.end()) {
-        if(it.key()->document() == view->document()) {
-          if(it.key()->attribute() == it->first)
-            it.key()->setAttribute(it->second); //Set the backed up attribute, if it wasn't changed yet
-          ignoreRange(it.key());
-          it = m_backups.erase(it);
-        } else {
-          ++it;
-        }
-      }
-      
-      changeHighlight( view, m_highlightedDeclarations[view].data(), false, false );
-    }
-
-    m_highlightedDeclarations.clear();
+  return 0;
 }
 
-void ContextBrowserPlugin::updateBrowserWidgetFor(View* view)
+void ContextBrowserPlugin::updateForView(View* view)
 {
-    bool keptHighlightedDeclaration = m_keepHighlightedDeclaration.contains(view) && m_highlightedDeclarations[view].data();
-    m_keepHighlightedDeclaration.remove(view);
-    
-    bool mouseHighlight =
-      (view->document()->url() == m_mouseHoverDocument) &&
-      (m_mouseHoverCursor.isValid());
-
-    SimpleCursor position;
-    if (mouseHighlight) {
-     position = m_mouseHoverCursor;
-    } else {
-     position = SimpleCursor(view->cursorPosition());
+    bool allowHighlight = true;
+    if(view->selection())
+    {
+      // If something is selected, we unhighlight everything, so that we don't conflict with the
+      // kate plugin that highlights occurences of the selected string, and also to reduce the
+      // overall amount of concurrent highlighting.
+      allowHighlight = false;
     }
-
-    ///First: Check whether there is a special language object
-    ///@todo Maybe make this optional, because it can be slow
-    ILanguage* pickedLanguage = 0;
-    bool foundSpecialObject = 0;
-    if(!keptHighlightedDeclaration)
-      foundSpecialObject  = findSpecialObject(view,position, pickedLanguage);
-
-    KDevelop::DUChainReadLocker lock( DUChain::lock(), 100 );
-    if(!lock.locked()) {
-      kDebug() << "Failed to lock du-chain in time";
+    
+    if(m_highlightedRanges[view].keep)
+    {
+      m_highlightedRanges[view].keep = false;
       return;
     }
+    
+    // Clear all highlighting
+    m_highlightedRanges.clear();
+    
+    // Re-highlight
+    ViewHighlights& highlights = m_highlightedRanges[view];
+    
+    KUrl url = view->document()->url();
+    IDocument* activeDoc = core()->documentController()->activeDocument();
+    
+    bool mouseHighlight = (url == m_mouseHoverDocument) && (m_mouseHoverCursor.isValid());
+    bool shouldUpdateBrowser = (mouseHighlight || (view->isActiveView() && activeDoc && activeDoc->textDocument() == view->document()));
+    
+    SimpleCursor highlightPosition;
+    if (mouseHighlight)
+     highlightPosition = m_mouseHoverCursor;
+    else
+     highlightPosition = SimpleCursor(view->cursorPosition());
 
-    if(!keptHighlightedDeclaration)
-      unHighlightAll();
-
-    TopDUContext* topContext = DUChainUtils::standardContextForUrl(view->document()->url());
-    if (!topContext) return;
-    DUContext* ctx = contextAt(position, topContext);
-    if (!ctx) return;
-    Declaration* foundDeclaration = 0;
+    ///Pick a language
+    ILanguage* language = 0;
     
-    //Only update the history if this context is around the text cursor
-    if(core()->documentController()->activeDocument() && position == SimpleCursor(view->cursorPosition()) && view->document() == core()->documentController()->activeDocument()->textDocument())
-    {
-      foreach(ContextBrowserView* contextView, m_views)
-        if(masterWidget(contextView) == masterWidget(view))
-          contextView->updateHistory(ctx, position);
-    }
-    
-    if(keptHighlightedDeclaration) {
-      foundDeclaration = m_highlightedDeclarations[view].data();
-      kDebug() << "keeping";
-    }
-    
-    if(!foundSpecialObject && !foundDeclaration)
-      foundDeclaration = findDeclaration(view, position, mouseHighlight);
-    
-    if( foundDeclaration ) {
-      m_lastHighlightedDeclaration = IndexedDeclaration(foundDeclaration);
-      m_highlightedDeclarations[view] = foundDeclaration;
-      changeHighlight( view, foundDeclaration, true, mouseHighlight );
+    if(ICore::self()->languageController()->languagesForUrl(url).isEmpty()) {
+      kDebug() << "found no language for document" << url;
+      return;
     }else{
-      m_highlightedDeclarations.remove(view);
+      language = ICore::self()->languageController()->languagesForUrl(url).front();
     }
     
-    bool addedWidget = false;
-    // try to add a declaration navigation widget
-    if(foundDeclaration || foundSpecialObject) {
-      IDocument* doc = core()->documentController()->activeDocument();
-      if(mouseHighlight || (view->isActiveView() && doc && doc->textDocument() == view->document())) {
-          addedWidget = foundSpecialObject ?
-            showSpecialObjectView(view, position, pickedLanguage, ctx) :
-            showDeclarationView(view, position, foundDeclaration, ctx);
+    ///Check whether there is a special language object to highlight (for example a macro)
+    
+    SimpleRange specialRange = language->languageSupport()->specialLanguageObjectRange(url, highlightPosition);
+    ContextBrowserView* updateBrowserView = shouldUpdateBrowser ?  browserViewForTextView(view) : 0;
+    
+    if(specialRange.isValid())
+    {
+      // Highlight a special language object
+      if(allowHighlight)
+      {
+        highlights.highlights << PersistentMovingRange::Ptr(new PersistentMovingRange(specialRange, IndexedString(url)));
+        highlights.highlights.back()->setAttribute(highlightedSpecialObjectAttribute());
+        highlights.highlights.back()->setZDepth(highlightingZDepth);
+      }
+      if(updateBrowserView)
+        updateBrowserView->setSpecialNavigationWidget(language->languageSupport()->specialLanguageObjectNavigationWidget(url, highlightPosition));
+    }else{
+      KDevelop::DUChainReadLocker lock( DUChain::lock(), 100 );
+      if(!lock.locked()) {
+        kDebug() << "Failed to lock du-chain in time";
+        return;
+      }
+      
+      TopDUContext* topContext = DUChainUtils::standardContextForUrl(view->document()->url());
+      if (!topContext)
+        return;
+      DUContext* ctx = contextForHighlightingAt(highlightPosition, topContext);
+      if (!ctx) 
+        return;
+      
+      //Only update the history if this context is around the text cursor
+      if(core()->documentController()->activeDocument() && highlightPosition == SimpleCursor(view->cursorPosition()) && view->document() == core()->documentController()->activeDocument()->textDocument())
+      {
+        foreach(ContextBrowserView* contextView, m_views)
+          if(masterWidget(contextView) == masterWidget(view))
+            contextView->updateHistory(ctx, highlightPosition);
+      }
+      
+      Declaration* foundDeclaration = findDeclaration(view, highlightPosition, mouseHighlight);
+      
+      if( foundDeclaration ) {
+        m_lastHighlightedDeclaration = highlights.declaration = IndexedDeclaration(foundDeclaration);
+        if(allowHighlight)
+          addHighlight( view, foundDeclaration );
+        
+        if(updateBrowserView)
+          updateBrowserView->setDeclaration(foundDeclaration, topContext);
+      }else{
+        if(updateBrowserView)
+          updateBrowserView->setContext(ctx);
       }
-    }
-
-    // if that failed, try to add a context navigation widget
-    if(!addedWidget && !mouseHighlight) {
-        showContextView(view, position, ctx);
     }
 }
 
 void ContextBrowserPlugin::updateViews()
 {
   foreach( View* view, m_updateViews ) {
-    updateBrowserWidgetFor(view);
+    updateForView(view);
   }
   m_updateViews.clear();
   m_useDeclaration = IndexedDeclaration();
@@ -750,12 +570,7 @@ void ContextBrowserPlugin::declarationSelectedInUI(DeclarationPointer decl)
 
 void ContextBrowserPlugin::parseJobFinished(KDevelop::ParseJob* job)
 {
-  if(job->duChain() && job->duChain()->smartRange()) {
-    KDevelop::DUChainReadLocker lock( DUChain::lock() );
-    registerAsRangeWatcher(job->duChain());
-  }
-  
-  for(QMap< View*, DeclarationPointer >::const_iterator it = m_highlightedDeclarations.constBegin(); it != m_highlightedDeclarations.constEnd(); ++it) {
+  for(QMap< View*, ViewHighlights >::iterator it = m_highlightedRanges.begin(); it != m_highlightedRanges.end(); ++it) {
     if(it.key()->document()->url() == job->document().toUrl()) {
       if(m_updateViews.isEmpty())
         m_updateTimer->start(highlightingTimeout);
@@ -763,50 +578,14 @@ void ContextBrowserPlugin::parseJobFinished(KDevelop::ParseJob* job)
       if(!m_updateViews.contains(it.key())) {
         kDebug() << "adding view for update";
         m_updateViews << it.key();
-        m_keepHighlightedDeclaration.insert(it.key());
+        
+        // Don't change the highlighted declaration after finished parse-jobs
+        (*it).keep = true;
       }
     }
   }
 }
 
-void ContextBrowserPlugin::registerAsRangeWatcher(KDevelop::DUChainBase* base)
-{
-  if(base->smartRange()) {
-    watchRange(base->smartRange());
-}
-}
-
-void ContextBrowserPlugin::registerAsRangeWatcher(KDevelop::DUContext* ctx)
-{
-  if(!ctx || !ctx->smartRange())
-    return;
-
-  QMutex* lockSmartMutex = 0;
-
-  if(!ctx->parentContext()) {
-    KTextEditor::SmartInterface* smart = dynamic_cast<KTextEditor::SmartInterface*>(ctx->smartRange()->document());
-    Q_ASSERT(smart);
-    lockSmartMutex = smart->smartMutex();
-  }
-  
-  QMutexLocker lock(lockSmartMutex);
-  
-  if(dynamic_cast<TopDUContext*>(ctx) && static_cast<TopDUContext*>(ctx)->parsingEnvironmentFile() && static_cast<TopDUContext*>(ctx)->parsingEnvironmentFile()->isProxyContext() && !ctx->importedParentContexts().isEmpty())
-    return registerAsRangeWatcher(ctx->importedParentContexts()[0].context(0));
-
-  foreach(Declaration* decl, ctx->localDeclarations())
-    registerAsRangeWatcher(decl);
-
-  for(int a = 0; a < ctx->usesCount(); ++a) {
-    KTextEditor::SmartRange* range = ctx->useSmartRange(a);
-    if(range)
-      watchRange(range);
-  }
-
-  foreach(DUContext* child, ctx->childContexts())
-    registerAsRangeWatcher(child);
-}
-
 void ContextBrowserPlugin::textDocumentCreated( KDevelop::IDocument* document )
 {
   Q_ASSERT(document->textDocument());
@@ -815,12 +594,6 @@ void ContextBrowserPlugin::textDocumentCreated( KDevelop::IDocument* document )
 
   foreach( View* view, document->textDocument()->views() )
     viewCreated( document->textDocument(), view );
-
-  KDevelop::DUChainReadLocker lock( DUChain::lock() );
-  QList<TopDUContext*> chains = DUChain::self()->chainsForDocument( document->url() );
-
-  foreach( TopDUContext* chain, chains )
-    registerAsRangeWatcher( chain );
 }
 
 void ContextBrowserPlugin::documentActivated( IDocument* doc )
@@ -833,10 +606,17 @@ void ContextBrowserPlugin::documentActivated( IDocument* doc )
 
 void ContextBrowserPlugin::viewDestroyed( QObject* obj )
 {
-  m_highlightedDeclarations.remove(static_cast<KTextEditor::View*>(obj));
+  m_highlightedRanges.remove(static_cast<KTextEditor::View*>(obj));
   m_updateViews.remove(static_cast<View*>(obj));
 }
 
+void ContextBrowserPlugin::selectionChanged( View* view )
+{
+  clearMouseHover();
+  m_updateViews.insert(view);
+  m_updateTimer->start(highlightingTimeout/2); // triggers updateViews()
+}
+
 void ContextBrowserPlugin::cursorPositionChanged( View* view, const KTextEditor::Cursor& newPosition )
 {
   if(view->document() == m_lastInsertionDocument && newPosition == m_lastInsertionPos)
@@ -844,9 +624,11 @@ void ContextBrowserPlugin::cursorPositionChanged( View* view, const KTextEditor:
     //Do not update the highlighting while typing
     m_lastInsertionDocument = 0;
     m_lastInsertionPos = KTextEditor::Cursor();
-    m_keepHighlightedDeclaration.insert(view);
+    if(m_highlightedRanges.contains(view))
+      m_highlightedRanges[view].keep = true;
   }else{
-    m_keepHighlightedDeclaration.remove(view);
+    if(m_highlightedRanges.contains(view))
+      m_highlightedRanges[view].keep = false;
   }
   clearMouseHover();
   m_updateViews.insert(view);
@@ -866,6 +648,7 @@ void ContextBrowserPlugin::viewCreated( KTextEditor::Document* , View* v )
   connect( v, SIGNAL(destroyed( QObject* )), this, SLOT( viewDestroyed( QObject* ) ) );
   disconnect( v->document(), SIGNAL(textInserted(KTextEditor::Document*,KTextEditor::Range)), this, SLOT(textInserted(KTextEditor::Document*,KTextEditor::Range)));
   connect( v->document(), SIGNAL(textInserted(KTextEditor::Document*,KTextEditor::Range)), this, SLOT(textInserted(KTextEditor::Document*,KTextEditor::Range)));
+  disconnect( v, SIGNAL(selectionChanged(KTextEditor::View*)), this, SLOT(selectionChanged(KTextEditor::View*)));
 
   KTextEditor::TextHintInterface *iface = dynamic_cast<KTextEditor::TextHintInterface*>(v);
   if( !iface )
@@ -899,7 +682,6 @@ void ContextBrowserPlugin::switchUse(bool forward)
 {
   if(core()->documentController()->activeDocument() && core()->documentController()->activeDocument()->textDocument() && core()->documentController()->activeDocument()->textDocument()->activeView()) {
     KTextEditor::Document* doc = core()->documentController()->activeDocument()->textDocument();
-    KDevelop::SimpleCursor c(doc->activeView()->cursorPosition());
 
 
     KDevelop::DUChainReadLocker lock( DUChain::lock() );
@@ -907,6 +689,9 @@ void ContextBrowserPlugin::switchUse(bool forward)
 
     if( chosen )
     {
+      SimpleCursor cCurrent(doc->activeView()->cursorPosition());
+      KDevelop::CursorInRevision c = chosen->transformToLocalRevision(cCurrent);
+      
       Declaration* decl = 0;
       //If we have a locked declaration, use that for jumping
       foreach(ContextBrowserView* view, m_views) {
@@ -916,7 +701,7 @@ void ContextBrowserPlugin::switchUse(bool forward)
       }
 
       if(!decl) //Try finding a declaration under the cursor
-        decl = DUChainUtils::itemUnderCursor(doc->url(), c);
+        decl = DUChainUtils::itemUnderCursor(doc->url(), cCurrent);
 
       if (decl && decl->kind() == Declaration::Alias) {
         AliasDeclaration* alias = dynamic_cast<AliasDeclaration*>(decl);
@@ -936,7 +721,7 @@ void ContextBrowserPlugin::switchUse(bool forward)
           target = FunctionDefinition::definition(decl);
         
           if(target && target != decl) {
-            SimpleCursor jumpTo = target->range().start;
+            SimpleCursor jumpTo = target->rangeInCurrentRevision().start;
             KUrl document = target->url().toUrl();
             lock.unlock();
             core()->documentController()->openDocument( document, cursorToRange(jumpTo)  );
@@ -963,11 +748,11 @@ void ContextBrowserPlugin::switchUse(bool forward)
             if(!usingFiles.isEmpty()) {
             TopDUContext* top = (forward ? usingFiles[0] : usingFiles.back()).data();
             if(top) {
-              QList<SimpleRange> useRanges = allUses(top, decl, true);
+              QList<RangeInRevision> useRanges = allUses(top, decl, true);
               qSort(useRanges);
               if(!useRanges.isEmpty()) {
                 KUrl url = top->url().toUrl();
-                SimpleRange selectUse = forward ? useRanges.first() : useRanges.back();
+                SimpleRange selectUse = chosen->transformFromLocalRevision(forward ? useRanges.first() : useRanges.back());
                 lock.unlock();
                 core()->documentController()->openDocument(url, cursorToRange(selectUse.start));
               }
@@ -976,7 +761,7 @@ void ContextBrowserPlugin::switchUse(bool forward)
           return;
         }
         //Check whether we are within a use
-        QList<SimpleRange> localUses = allUses(chosen, decl, true);
+        QList<RangeInRevision> localUses = allUses(chosen, decl, true);
         qSort(localUses);
         for(int a = 0; a < localUses.size(); ++a) {
           int nextUse = (forward ? a+1 : a-1);
@@ -1039,7 +824,7 @@ void ContextBrowserPlugin::switchUse(bool forward)
                     decl = definition;
                 }
                 KUrl u(decl->url().str());
-                SimpleRange range = decl->range();
+                SimpleRange range = decl->rangeInCurrentRevision();
                 range.end = range.start;
                 lock.unlock();
                 core()->documentController()->openDocument(u, range.textRange());
@@ -1049,11 +834,11 @@ void ContextBrowserPlugin::switchUse(bool forward)
                 
                 KUrl u(nextTop->url().str());
                 
-                QList<SimpleRange> nextTopUses = allUses(nextTop, decl, true);
+                QList<RangeInRevision> nextTopUses = allUses(nextTop, decl, true);
                 qSort(nextTopUses);
                 
                 if(!nextTopUses.isEmpty()) {
-                  SimpleRange range = forward ? nextTopUses.front() : nextTopUses.back();
+                  SimpleRange range =  chosen->transformFromLocalRevision(forward ? nextTopUses.front() : nextTopUses.back());
                   range.end = range.start;
                   lock.unlock();
                   core()->documentController()->openDocument(u, range.textRange());
@@ -1065,7 +850,7 @@ void ContextBrowserPlugin::switchUse(bool forward)
             }
           }else{
               KUrl url(chosen->url().str());
-              SimpleRange range = localUses[nextUse];
+              SimpleRange range = chosen->transformFromLocalRevision(localUses[nextUse]);
               range.end = range.start;
               lock.unlock();
               core()->documentController()->openDocument(url, range.textRange());
