@@ -46,6 +46,7 @@ private slots:
     void activateDeps( int );
     void checkActions( const QItemSelection& , const QItemSelection& );
     void depEdited( const QString& );
+    void configureEnv();
 };
 
 class NativeAppLauncher : public KDevelop::ILauncher
@@ -74,6 +75,7 @@ public:
  */
 class NativeAppConfigType : public KDevelop::LaunchConfigurationType
 {
+Q_OBJECT
 public:
     NativeAppConfigType();
 
@@ -87,8 +89,11 @@ public:
                                    KDevelop::ProjectBaseItem* item ) const;
     void configureLaunchFromCmdLineArguments ( KConfigGroup cfg,
                                                const QStringList& args ) const;
+    QMenu* launcherSuggestions();
 private:
     QList<KDevelop::LaunchConfigurationPageFactory*> factoryList;
+public slots:
+    void suggestionTriggered();
 };
 #endif
 
