@@ -101,7 +101,7 @@ void ProjectLoadTest::addRemoveFiles()
 
     KDevelop::ICore::self()->projectController()->openProject(p.second);
     QTest::qWait(500);
- 
+
     KDevelop::IProject* project = KDevelop::ICore::self()->projectController()->projects().first();
     Q_ASSERT(project->projectFileUrl() == p.second);
 
@@ -119,7 +119,7 @@ void ProjectLoadTest::addRemoveFiles()
 
     QCOMPARE(project->filesForUrl(p.first+"/blub"+QString::number(50)).count(), 1);
     KDevelop::ProjectFileItem* file = project->filesForUrl(p.first+"/blub"+QString::number(50)).first();
-    project->projectFileManager()->removeFile( file ); //message box has to be accepted manually :(
+    project->projectFileManager()->removeFilesAndFolders(QList<KDevelop::ProjectBaseItem*>() << file ); //message box has to be accepted manually :(
     for (int i=51; i<100; ++i) {
         QFile f2(p.first+"/blub"+QString::number(i));
         f2.remove();
@@ -162,8 +162,8 @@ void _writeRandomStructure(QString path, int files)
 void fillProject(int filesPerDir, int Dirs, const QPair<QString, KUrl> project, bool wait)
 {
     QDir(project.first).mkdir("foou");
-    _writeRandomStructure(project.first+"/foou", filesPerDir);
-    for(int i=0; i < Dirs; ++i) {
+    _writeRandomStructure(project.first+"/foou", 50);
+    for(int i=0; i < 100; ++i) {
         exec("bash -c \"cp -r foou foox"+QString::number(i)+" > /dev/null 2>&1 & \"");
         if (wait) {
             QTest::qWait(100);
@@ -197,12 +197,9 @@ void ProjectLoadTest::addMultipleJobs()
     fillProject(10, 25, p1, false);
     QPair<QString, KUrl> p2 = makeProject();
     fillProject(10, 25, p2, false);
-    QPair<QString, KUrl> p3 = makeProject();
-    fillProject(10, 25, p3, false);
 
     KDevelop::ICore::self()->projectController()->openProject(p1.second);
     KDevelop::ICore::self()->projectController()->openProject(p2.second);
-    KDevelop::ICore::self()->projectController()->openProject(p3.second);
 
     QTest::qWait(2000);
 
