@@ -48,14 +48,13 @@
 #include <interfaces/iruncontroller.h>
 #include "stashmanagerdialog.h"
 #include <KMessageBox>
+#include "gitjob.h"
 
 K_PLUGIN_FACTORY(KDevGitFactory, registerPlugin<GitPlugin>(); )
 K_EXPORT_PLUGIN(KDevGitFactory(KAboutData("kdevgit","kdevgit",ki18n("Git"),"0.1",ki18n("A plugin to support git version control systems"), KAboutData::License_GPL)))
 
 using namespace KDevelop;
 
-typedef DVcsJob GitJob;
-
 namespace
 {
     
@@ -66,7 +65,7 @@ QDir dotGitDirectory(const KUrl& dirPath)
     
     static const QString gitDir(".git");
     while (!dir.exists(gitDir) && dir.cdUp()) {} // cdUp, until there is a sub-directory called .git
-    
+
     return dir;
 }
 
@@ -291,7 +290,7 @@ VcsJob* GitPlugin::add(const KUrl::List& localLocations, KDevelop::IBasicVersion
     if (localLocations.empty())
         return errorsFound(i18n("Did not specify the list of files"), OutputJob::Verbose);
 
-    DVcsJob* job = new DVcsJob(urlDir(localLocations), this);
+    DVcsJob* job = new GitJob(dotGitDirectory(localLocations.front()), this);
     *job << "git" << "add" << "--" << (recursion == IBasicVersionControl::Recursive ? localLocations : preventRecursion(localLocations));
     return job;
 }
@@ -301,11 +300,17 @@ KDevelop::VcsJob* GitPlugin::status(const KUrl::List& localLocations, KDevelop::
     if (localLocations.empty())
         return errorsFound(i18n("Did not specify the list of files"), OutputJob::Verbose);
 
-    DVcsJob* job = new DVcsJob(urlDir(localLocations), this, OutputJob::Silent);
-    *job << "git" << "status" << "--porcelain" << "--";
-    *job << (recursion == IBasicVersionControl::Recursive ? localLocations : preventRecursion(localLocations));
+    DVcsJob* job = new GitJob(urlDir(localLocations), this, OutputJob::Silent);
+    
+    if(m_oldVersion) {
+        *job << "git" << "ls-files" << "-t" << "-m" << "-c" << "-o" << "-d" << "-k" << "--directory";
+        connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitStatusOutput_old(KDevelop::DVcsJob*)));
+    } else {
+        *job << "git" << "status" << "--porcelain";
+        connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitStatusOutput(KDevelop::DVcsJob*)));
+    }
+    *job << "--" << (recursion == IBasicVersionControl::Recursive ? localLocations : preventRecursion(localLocations));
 
-    connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), SLOT(parseGitStatusOutput(KDevelop::DVcsJob*)));
     return job;
 }
 
@@ -314,7 +319,7 @@ VcsJob* GitPlugin::diff(const KUrl& fileOrDirectory, const KDevelop::VcsRevision
 {
     //TODO: control different types
     
-    DVcsJob* job = new DVcsJob(urlDir(fileOrDirectory), this);
+    DVcsJob* job = new GitJob(dotGitDirectory(fileOrDirectory), this);
     *job << "git" << "diff" << "--no-prefix";
     QString revstr = revisionInterval(srcRevision, dstRevision);
     if(!revstr.isEmpty())
@@ -331,7 +336,7 @@ VcsJob* GitPlugin::revert(const KUrl::List& localLocations, IBasicVersionControl
     if(localLocations.isEmpty() )
         return errorsFound(i18n("Could not revert changes"), OutputJob::Verbose);
     
-    DVcsJob* job = new DVcsJob(urlDir(localLocations), this);
+    DVcsJob* job = new GitJob(dotGitDirectory(localLocations.front()), this);
     *job << "git" << "checkout" << "--";
     *job << (recursion == IBasicVersionControl::Recursive ? localLocations : preventRecursion(localLocations));
     
@@ -415,7 +420,7 @@ VcsJob* GitPlugin::log(const KUrl& localLocation,
 
 VcsJob* GitPlugin::log(const KUrl& localLocation, const KDevelop::VcsRevision& rev, unsigned long int limit)
 {
-    DVcsJob* job = new DVcsJob(urlDir(localLocation), this, KDevelop::OutputJob::Silent);
+    DVcsJob* job = new GitJob(dotGitDirectory(localLocation), this, KDevelop::OutputJob::Silent);
     *job << "git" << "log" << "--date=raw" << toRevisionName(rev, QString());
     if(limit>0)
         *job << QString("-%1").arg(limit);
@@ -427,7 +432,7 @@ VcsJob* GitPlugin::log(const KUrl& localLocation, const KDevelop::VcsRevision& r
 
 KDevelop::VcsJob* GitPlugin::annotate(const KUrl &localLocation, const KDevelop::VcsRevision&)
 {
-    DVcsJob* job = new DVcsJob(urlDir(localLocation), this, KDevelop::OutputJob::Silent);
+    DVcsJob* job = new GitJob(dotGitDirectory(localLocation), this, KDevelop::OutputJob::Silent);
     *job << "git" << "blame" << "--porcelain";
     *job << "--" << localLocation;
     connect(job, SIGNAL(readyForParsing(KDevelop::DVcsJob*)), this, SLOT(parseGitBlameOutput(KDevelop::DVcsJob*)));
@@ -458,7 +463,7 @@ void GitPlugin::parseGitBlameOutput(DVcsJob *job)
         QString name = it->left(it->indexOf(' '));
         QString value = it->right(it->size()-name.size()-1);
         
-        qDebug() << "last line" << *it;
+        kDebug() << "last line" << *it;
         if(name=="author")
             annotation->setAuthor(value);
         else if(name=="author-mail") {} //TODO: do smth with the e-mail?
@@ -478,7 +483,7 @@ void GitPlugin::parseGitBlameOutput(DVcsJob *job)
             QStringList values = value.split(' ');
             
             VcsRevision rev;
-            rev.setRevisionValue(name, KDevelop::VcsRevision::GlobalNumber);
+            rev.setRevisionValue(name.left(8), KDevelop::VcsRevision::GlobalNumber);
             
             skipNext = definedRevisions.contains(name);
             
@@ -498,7 +503,7 @@ DVcsJob* GitPlugin::switchBranch(const QString &repository, const QString &branc
     QDir d(repository);
     
     if(hasModifications(d) && KMessageBox::questionYesNo(0, i18n("There are pending changes, do you want to stash them first?"))==KMessageBox::Yes) {
-        QScopedPointer<DVcsJob> stash(gitStash(urlDir(m_urls), QStringList(), KDevelop::OutputJob::Verbose));
+        QScopedPointer<DVcsJob> stash(gitStash(d, QStringList(), KDevelop::OutputJob::Verbose));
         stash->exec();
     }
     
@@ -889,20 +894,49 @@ void GitPlugin::parseGitDiffOutput(DVcsJob* job)
     job->setResults(qVariantFromValue(diff));
 }
 
-// static VcsStatusInfo::State lsfilesToState(char id)
-// {
-//     switch(id) {
-//         case 'H': return VcsStatusInfo::ItemUpToDate; //Cached
-//         case 'S': return VcsStatusInfo::ItemUpToDate; //Skip work tree
-//         case 'M': return VcsStatusInfo::ItemHasConflicts; //unmerged
-//         case 'R': return VcsStatusInfo::ItemDeleted; //removed/deleted
-//         case 'C': return VcsStatusInfo::ItemModified; //modified/changed
-//         case 'K': return VcsStatusInfo::ItemDeleted; //to be killed
-//         case '?': return VcsStatusInfo::ItemUnknown; //other
-//     }
-//     Q_ASSERT(false);
-//     return VcsStatusInfo::ItemUnknown;
-// }
+static VcsStatusInfo::State lsfilesToState(char id)
+{
+    switch(id) {
+        case 'H': return VcsStatusInfo::ItemUpToDate; //Cached
+        case 'S': return VcsStatusInfo::ItemUpToDate; //Skip work tree
+        case 'M': return VcsStatusInfo::ItemHasConflicts; //unmerged
+        case 'R': return VcsStatusInfo::ItemDeleted; //removed/deleted
+        case 'C': return VcsStatusInfo::ItemModified; //modified/changed
+        case 'K': return VcsStatusInfo::ItemDeleted; //to be killed
+        case '?': return VcsStatusInfo::ItemUnknown; //other
+    }
+    Q_ASSERT(false);
+    return VcsStatusInfo::ItemUnknown;
+}
+
+void GitPlugin::parseGitStatusOutput_old(DVcsJob* job)
+{
+    QStringList outputLines = job->output().split('\n', QString::SkipEmptyParts);
+    
+    KUrl d = job->directory().absolutePath();
+    QMap<KUrl, VcsStatusInfo::State> allStatus;
+    foreach(const QString& line, outputLines) {
+        VcsStatusInfo::State status = lsfilesToState(line[0].toAscii());
+        
+        KUrl url = d;
+        url.addPath(line.right(line.size()-2));
+        
+        allStatus[url] = status;
+    }
+    
+    QVariantList statuses;
+    QMap< KUrl, VcsStatusInfo::State >::const_iterator it = allStatus.constBegin(), itEnd=allStatus.constEnd();
+    for(; it!=itEnd; ++it) {
+        
+        VcsStatusInfo status;
+        status.setUrl(it.key());
+        status.setState(it.value());
+        
+        statuses.append(qVariantFromValue<VcsStatusInfo>(status));
+    }
+    
+    job->setResults(statuses);
+}
 
 void GitPlugin::parseGitStatusOutput(DVcsJob* job)
 {
