@@ -27,12 +27,15 @@ Q_OBJECT
 public:
     OpenProjectPage( const KUrl& startUrl, QWidget* parent = 0 );
     QMap<QString,QStringList> projectFilters() const;
+    void setUrl(const KUrl& url);
+    
 signals:
     void urlSelected( const KUrl& );
 private slots:
     void highlightFile(const QString&);
     void opsEntered(const KUrl& item );
     void comboTextChanged(const QString& );
+    void dirChanged(const KUrl& url);
 private:
     
     KUrl getAbsoluteUrl( const QString& ) const;
