@@ -74,6 +74,7 @@ Boston, MA 02110-1301, USA.
 #include <kio/job.h>
 #include "sessioncontroller.h"
 #include "session.h"
+#include <QApplication>
 
 namespace KDevelop
 {
@@ -166,11 +167,8 @@ public:
         {
             IPlugin* plugin = *it;
             IProjectFileManager* iface = plugin->extension<KDevelop::IProjectFileManager>();
-            const KPluginInfo info = m_core->pluginController()->pluginInfo( plugin );
-            if (info.property("X-KDevelop-Category").toString() != "Project")
-                continue;
             if( !iface || plugin == project->managerPlugin() )
-                pluginnames << info.pluginName();
+                pluginnames << m_core->pluginController()->pluginInfo( plugin ).pluginName();
         }
 
         return pluginnames;
@@ -309,42 +307,93 @@ bool projectFileExists( const KUrl& u )
     }
 }
 
+bool equalProjectFile( const QString& configPath, OpenProjectDialog* dlg )
+{
+    KSharedConfig::Ptr cfg = KSharedConfig::openConfig( configPath, KConfig::SimpleConfig );
+    KConfigGroup grp = cfg->group( "Project" );
+    QString defaultName = dlg->projectFileUrl().upUrl().fileName();
+    return (grp.readEntry( "Name", QString() ) == dlg->projectName() || dlg->projectName() == defaultName) &&
+           grp.readEntry( "Manager", QString() ) == dlg->projectManager();
+}
+
 KUrl ProjectDialogProvider::askProjectConfigLocation(bool fetch, const KUrl& startUrl)
 {
     Q_ASSERT(d);
     OpenProjectDialog dlg( fetch, startUrl, Core::self()->uiController()->activeMainWindow() );
     if(dlg.exec() == QDialog::Rejected)
         return KUrl();
-    
+
     KUrl projectFileUrl = dlg.projectFileUrl();
     kDebug() << "selected project:" << projectFileUrl << dlg.projectName() << dlg.projectManager();
-    if( !projectFileExists( projectFileUrl ) )
+    if( projectFileExists( projectFileUrl ) )
     {
+        // check whether config is equal
+        bool shouldAsk = true;
         if( projectFileUrl.isLocalFile() )
         {
-            bool ok = writeNewProjectFile( KSharedConfig::openConfig( projectFileUrl.toLocalFile(), KConfig::SimpleConfig ),
-                            dlg.projectName(),
-                            dlg.projectManager() );
-            if (!ok)
-                return KUrl();
-        } else
-        {
-            KTemporaryFile tmp;
-            tmp.open();
-            bool ok = writeNewProjectFile( KSharedConfig::openConfig( tmp.fileName(), KConfig::SimpleConfig ),
-                            dlg.projectName(),
-                            dlg.projectManager() );
-            if (!ok)
-                return KUrl();
-
-            ok = KIO::NetAccess::upload( tmp.fileName(), projectFileUrl, Core::self()->uiControllerInternal()->defaultMainWindow() );
-            if (!ok) {
-                KMessageBox::error(d->m_core->uiControllerInternal()->defaultMainWindow(),
-                    i18n("Unable to create configuration file %1", projectFileUrl.url()));
-                return KUrl();
+            shouldAsk = !equalProjectFile( projectFileUrl.toLocalFile(), &dlg );
+        } else {
+            QString tmpFile;
+            if ( KIO::NetAccess::download( projectFileUrl, tmpFile, qApp->activeWindow() ) ) {
+                shouldAsk = !equalProjectFile( tmpFile, &dlg );
+                QFile::remove(tmpFile);
+            } else {
+                shouldAsk = false;
             }
         }
+
+        if ( shouldAsk )
+        {
+            KGuiItem yes = KStandardGuiItem::yes();
+            yes.setText(i18n("Override"));
+            yes.setToolTip(i18n("Continue to open the project and use the just provided project configuration."));
+            yes.setIcon(KIcon());
+            KGuiItem no = KStandardGuiItem::no();
+            no.setText(i18n("Open Existing File"));
+            no.setToolTip(i18n("Continue to open the project but use the existing project configuration."));
+            no.setIcon(KIcon());
+            KGuiItem cancel = KStandardGuiItem::cancel();
+            cancel.setToolTip(i18n("Cancel and don't open the project."));
+            int ret = KMessageBox::questionYesNoCancel(qApp->activeWindow(),
+                i18n("There already exists a project configuration file at %1.\n"
+                     "Do you want to override it or open the existing file?", projectFileUrl.pathOrUrl()),
+                i18n("Override existing project configuration"), yes, no, cancel );
+            if ( ret == KMessageBox::No )
+            {
+                // no: reuse existing project file
+                return projectFileUrl;
+            } else if ( ret == KMessageBox::Cancel )
+            {
+                return KUrl();
+            } // else fall through and write new file
+        }
     }
+
+    if( projectFileUrl.isLocalFile() )
+    {
+        bool ok = writeNewProjectFile( KSharedConfig::openConfig( projectFileUrl.toLocalFile(), KConfig::SimpleConfig ),
+                        dlg.projectName(),
+                        dlg.projectManager() );
+        if (!ok)
+            return KUrl();
+    } else
+    {
+        KTemporaryFile tmp;
+        tmp.open();
+        bool ok = writeNewProjectFile( KSharedConfig::openConfig( tmp.fileName(), KConfig::SimpleConfig ),
+                        dlg.projectName(),
+                        dlg.projectManager() );
+        if (!ok)
+            return KUrl();
+
+        ok = KIO::NetAccess::upload( tmp.fileName(), projectFileUrl, Core::self()->uiControllerInternal()->defaultMainWindow() );
+        if (!ok) {
+            KMessageBox::error(d->m_core->uiControllerInternal()->defaultMainWindow(),
+                i18n("Unable to create configuration file %1", projectFileUrl.url()));
+            return KUrl();
+        }
+    }
+
     return projectFileUrl;
 }
 
@@ -729,26 +778,17 @@ void ProjectController::initializePluginCleanup(IProject* proj)
 
 void ProjectController::closeProject(IProject* proj_)
 {
-    if (!proj_)
+    if(!proj_ || d->m_projects.indexOf(proj_) == -1)
     {
         return;
     }
-
-    // loading might have failed
-    d->m_currentlyOpening.removeAll(proj_->projectFileUrl());
-
-    if(d->m_projects.indexOf(proj_) == -1)
-    {
-        return;
-    }
-
+    
     Project* proj = dynamic_cast<KDevelop::Project*>( proj_ );
     if( !proj ) 
     {
         kWarning() << "Unknown Project subclass found!";
         return;
     }
-    
     d->m_projects.removeAll(proj);
     emit projectClosing(proj);
     //Core::self()->saveSettings();     // The project file is being closed.
@@ -832,9 +872,23 @@ KUrl ProjectController::projectsBaseDirectory() const
                                      KUrl( QDir::homePath()+"/projects" ) );
 }
 
-QString ProjectController::prettyFileName(KUrl url, FormattingOptions format) const
+QString ProjectController::prettyFilePath(KUrl url, FormattingOptions format) const
 {
     IProject* project = Core::self()->projectController()->findProjectForUrl(url);
+    
+    if(!project)
+    {
+        // Find a project with the correct base directory at least
+        foreach(IProject* candidateProject, Core::self()->projectController()->projects())
+        {
+            if(candidateProject->folder().isParentOf(url))
+            {
+                project = candidateProject;
+                break;
+            }
+        }
+    }
+    
     QString prefixText = url.upUrl().pathOrUrl(KUrl::AddTrailingSlash);
     if (project) {
         if (format == FormatHtml) {
@@ -847,6 +901,12 @@ QString ProjectController::prettyFileName(KUrl url, FormattingOptions format) co
             relativePath = relativePath.mid(2);
         prefixText += relativePath;
     }
+    return prefixText;
+}
+
+QString ProjectController::prettyFileName(KUrl url, FormattingOptions format) const
+{
+    QString prefixText = prettyFilePath( url, format );
     if (format == FormatHtml) {
         return prefixText + "<b>" + url.fileName() + "</b>";
     } else {
