@@ -377,7 +377,10 @@ KJob* RunController::execute(const QString& runMode, ILaunchConfiguration* launc
     ILauncher* launcher = run->type()->launcherForId( launcherId );
 
     if( !launcher ) {
-        kWarning() << i18n("Launcher could not be found for the name '%1'. Check the launch configuration.", launcherId );
+        KMessageBox::error(
+            qApp->activeWindow(), 
+            i18n("The current launch configuration does not support the '%1' mode.", runMode),
+            "");
         return 0;
     }
 
@@ -586,13 +589,7 @@ void KDevelop::RunController::checkState()
 
 void KDevelop::RunController::stopAllProcesses()
 {
-    // composite jobs might remove child jobs, see also:
-    // https://bugs.kde.org/show_bug.cgi?id=258904
-    // foreach already iterates over a copy
     foreach (KJob* job, d->jobs.keys()) {
-        // now we check the real list whether it was deleted
-        if (!d->jobs.contains(job))
-            continue;
         if (job->capabilities() & KJob::Killable)
             job->kill(KJob::EmitResult);
     }
@@ -830,7 +827,7 @@ ContextMenuExtension RunController::contextMenuExtension ( Context* ctx )
 {
     delete d->launchAsMapper;
     d->launchAsMapper = new QSignalMapper( this );
-    kDebug() << "connected launchmapper:" << connect( d->launchAsMapper, SIGNAL( mapped( int ) ), SLOT( launchAs( int ) ) );
+    connect( d->launchAsMapper, SIGNAL( mapped( int ) ), SLOT( launchAs( int ) ) );
     d->launchAsInfo.clear();
     d->contextItem = 0;
     ContextMenuExtension ext;
@@ -860,7 +857,7 @@ ContextMenuExtension RunController::contextMenuExtension ( Context* ctx )
                         act->setText( type->name() );
                         kDebug() << "Setting up mapping for:" << i << "for action" << act->text() << "in mode" << mode->name();
                         d->launchAsMapper->setMapping( act, i );
-                        kDebug() << "action connected:" << connect( act, SIGNAL( triggered() ), d->launchAsMapper, SLOT( map() ) );
+                        connect( act, SIGNAL( triggered() ), d->launchAsMapper, SLOT( map() ) );
                         menu->addAction(act);
                         i++;
                     }
