@@ -50,6 +50,7 @@
 #include <kactioncollection.h>
 #include <ktexteditor/view.h>
 #include "workingsetcontroller.h"
+#include "workingsets/workingset.h"
 
 namespace KDevelop {
 
@@ -58,7 +59,8 @@ public:
     UiControllerPrivate(UiController *controller)
     : areasRestored(false), m_controller(controller)
     {
-        Core::self()->workingSetControllerInternal()->initializeController(m_controller);
+        if (Core::self()->workingSetControllerInternal())
+            Core::self()->workingSetControllerInternal()->initializeController(m_controller);
 
         QMap<QString, Sublime::Position> desired;
 
@@ -148,6 +150,10 @@ public:
         return m_factory->create(parent);
     }
 
+    virtual QList< QAction* > contextMenuActions(QWidget* viewWidget) const
+    {
+        return m_factory->contextMenuActions( viewWidget );
+    }
 
     QList<QAction*> toolBarActions( QWidget* viewWidget ) const
     {
@@ -179,33 +185,6 @@ UiController::UiController(Core *core)
     connect( QApplication::instance(),
              SIGNAL( focusChanged( QWidget*, QWidget* ) ),
             this, SLOT( widgetChanged( QWidget*, QWidget* ) ) );
-
-    KActionCollection * actions = defaultMainWindow()->actionCollection();
-
-    KAction* assistantaction1 = actions->addAction("assistant_action_1");
-    assistantaction1->setText( i18n("&Assistant Action 1") );
-    assistantaction1->setShortcut( Qt::ALT | Qt::Key_1 );
-    connect(assistantaction1, SIGNAL(triggered(bool)), this, SLOT(assistantAction1()));
-
-    KAction* assistantaction2 = actions->addAction("assistant_action_2");
-    assistantaction2->setText( i18n("&Assistant Action 2") );
-    assistantaction2->setShortcut( Qt::ALT | Qt::Key_2 );
-    connect(assistantaction2, SIGNAL(triggered(bool)), this, SLOT(assistantAction2(bool)));
-
-    KAction* assistantaction3 = actions->addAction("assistant_action_3");
-    assistantaction3->setText( i18n("&Assistant Action 3") );
-    assistantaction3->setShortcut( Qt::ALT | Qt::Key_3 );
-    connect(assistantaction3, SIGNAL(triggered(bool)), this, SLOT(assistantAction3(bool)));
-
-    KAction* assistantaction4 = actions->addAction("assistant_action_4");
-    assistantaction4->setText( i18n("&Assistant Action 4") );
-    assistantaction4->setShortcut( Qt::ALT | Qt::Key_4 );
-    connect(assistantaction4, SIGNAL(triggered(bool)), this, SLOT(assistantAction4(bool)));
-
-    KAction* assistantactionhide = actions->addAction("assistant_action_hide");
-    assistantactionhide->setText( i18n("&Hide Assistant") );
-    assistantactionhide->setShortcut( Qt::ALT | Qt::Key_0 );
-    connect(assistantactionhide, SIGNAL(triggered(bool)), this, SLOT(assistantHide()));
 }
 
 UiController::~UiController()
@@ -433,14 +412,19 @@ KParts::MainWindow *UiController::activeMainWindow()
 void UiController::saveArea(Sublime::Area * area, KConfigGroup & group)
 {
     area->save(group);
-    Core::self()->workingSetControllerInternal()->getWorkingSet(area->workingSet())->saveFromArea(area, area->rootIndex());
+    if (!area->workingSet().isEmpty()) {
+        WorkingSet* set = Core::self()->workingSetControllerInternal()->getWorkingSet(area->workingSet());
+        set->saveFromArea(area, area->rootIndex());
+    }
 }
 
 void UiController::loadArea(Sublime::Area * area, const KConfigGroup & group)
 {
     area->load(group);
-    WorkingSet* set = Core::self()->workingSetControllerInternal()->getWorkingSet(area->workingSet());
-    Q_ASSERT(set->isConnected(area));
+    if (!area->workingSet().isEmpty()) {
+        WorkingSet* set = Core::self()->workingSetControllerInternal()->getWorkingSet(area->workingSet());
+        Q_ASSERT(set->isConnected(area));
+    }
 }
 
 void UiController::saveAllAreas(KSharedConfig::Ptr config)
@@ -650,25 +634,6 @@ const QMap< IToolViewFactory*, Sublime::ToolDocument* >& UiController::factoryDo
     return d->factoryDocuments;
 }
 
-void UiController::assistantAction1() {
-    if(d->currentShownAssistant)
-        d->currentShownAssistant->executeAction1();
-}
-
-void UiController::assistantAction2(bool) {
-    if(d->currentShownAssistant)
-        d->currentShownAssistant->executeAction2();
-}
-
-void UiController::assistantAction3(bool) {
-    if(d->currentShownAssistant)
-        d->currentShownAssistant->executeAction3();
-}
-void UiController::assistantAction4(bool) {
-    if(d->currentShownAssistant)
-        d->currentShownAssistant->executeAction4();
-}
-
 void UiController::assistantHide() {
     if(d->currentShownAssistant)
         hideAssistant(d->currentShownAssistant->assistant());
@@ -679,7 +644,6 @@ void UiController::assistantActionsChanged() {
         popUpAssistant(d->currentShownAssistant->assistant());
 }
 
-
 }
 
 
