@@ -43,6 +43,7 @@
 #include <qpointer.h>
 #include <QEvent>
 #include <QKeyEvent>
+#include "urldocument.h"
 
 namespace Sublime {
 
@@ -183,7 +184,7 @@ Container::Container(QWidget *parent)
     l->addWidget(d->stack);
 
     connect(d->tabBar, SIGNAL(currentChanged(int)), this, SLOT(widgetActivated(int)));
-    connect(d->tabBar, SIGNAL(closeRequest(int)), this, SLOT(closeRequest(int)));
+    connect(d->tabBar, SIGNAL(tabCloseRequested(int)), this, SLOT(closeRequest(int)));
     connect(d->tabBar, SIGNAL(tabMoved(int,int)), this, SLOT(tabMoved(int, int)));
     connect(d->tabBar, SIGNAL(wheelDelta(int)), this, SLOT(wheelScroll(int)));
     connect(d->tabBar, SIGNAL(contextMenu(int,QPoint)), this, SLOT(contextMenu(int,QPoint)));
@@ -307,7 +308,21 @@ void Container::documentTitleChanged(Sublime::Document* doc)
     QMapIterator<QWidget*, View*> it = d->viewForWidget;
     while (it.hasNext()) {
         if (it.next().value()->document() == doc) {
-            d->fileNameCorner->setText( doc->title() );
+            QString txt = doc->title();
+            //TODO: Maybe add new virtual in Document to support supplying this
+            // extended information from subclasses like IDocument which can use
+            // the rest of the kdevplatform API
+            UrlDocument* udoc = dynamic_cast<UrlDocument*>( doc );
+            if( udoc ) {
+                QString pretty;
+                if( udoc->url().isLocalFile() ) {
+                    pretty = udoc->url().toLocalFile();
+                } else {
+                    pretty = udoc->url().prettyUrl();
+                }
+                txt = txt + " (" + pretty + ")";
+            }
+            d->fileNameCorner->setText( txt );
             int tabIndex = d->stack->indexOf(it.key());
             if (tabIndex != -1) {
                 d->tabBar->setTabText(tabIndex, doc->title());
@@ -339,7 +354,8 @@ void Container::setCurrentWidget(QWidget* w)
     if (View *view = d->viewForWidget[w])
     {
         statusChanged(view);
-        d->fileNameCorner->setText(view->document()->title());
+        statusIconChanged( view->document() );
+        documentTitleChanged( view->document() );
     }
 }
 
@@ -359,8 +375,13 @@ void Sublime::Container::removeWidget(QWidget *w)
         int widgetIdx = d->stack->indexOf(w);
         d->stack->removeWidget(w);
         d->tabBar->removeTab(widgetIdx);
-        if (d->tabBar->currentIndex() != -1)
-            d->fileNameCorner->setText(d->tabBar->tabText(d->tabBar->currentIndex()));
+        if (d->tabBar->currentIndex() != -1) {
+            QWidget* w = widget( d->tabBar->currentIndex() );
+            if( w ) {
+                statusIconChanged( d->viewForWidget[w]->document() );
+                documentTitleChanged( d->viewForWidget[w]->document() );
+            }
+        }
         View* view = d->viewForWidget.take(w);
         if (view)
         {
@@ -439,8 +460,9 @@ void Container::contextMenu( int currentTab, QPoint pos )
             // activate last tab
             widgetActivated(count() - 1);
             // close all
-            while ( count() ) {
-                closeRequest(widget(0));
+            QList<QWidget*> tabs;
+            for ( int i = 0; i < count(); ++i ) {
+                closeRequest(widget(i));
             }
         } // else the action was handled by someone else
     }
